/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.examples.onlineSim.timeVaryingTraffic;

import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.interfaces.simulation.IEventProcessor;
import com.net2plan.interfaces.simulation.SimAction;
import com.net2plan.interfaces.simulation.SimEvent;
import com.net2plan.utils.Triple;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * This algorithm allocates the traffic over the existing routes. Link
 * over-subscription may happen.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @version 1.1, May 2015
 */
public class TVSim_AA_basicTrafficAllocationAlgorithm extends IEventProcessor
{
	private Map<Long, Double> routeCarriedTrafficFractionMap;
	
	@Override
	public String getDescription()
	{
		return "This algorithm allocates the traffic over the existing routes. Link over-subscription may happen";
	}

	@Override
	public List<Triple<String, String, String>> getParameters()
	{
		return new LinkedList<Triple<String, String, String>>();
	}

	@Override
	public void initialize(NetPlan initialNetPlan, NetPlan currentNetPlan, Map<String, String> algorithmParameters, Map<String, String> simulationParameters, Map<String, String> net2planParameters)
	{
		routeCarriedTrafficFractionMap = initialNetPlan.getRouteCarriedTrafficFractionMap();
	}

	@Override
	public void processEvent(NetPlan currentNetPlan, SimEvent event)
	{
		List<SimAction> actions = event.getEventActionList();
		for(SimAction action : actions)
		{
			if (action.getActionType() == SimAction.ActionType.DEMAND_MODIFIED)
			{
				long demandId = action.getDemandModified_demandId();
				double h_d = currentNetPlan.getDemandOfferedTraffic(demandId);
				
				Set<Long> routeIds_thisDemand = currentNetPlan.getDemandRoutes(demandId);
				for(long routeId : routeIds_thisDemand)
				{
					double f_p = routeCarriedTrafficFractionMap.get(routeId);
					currentNetPlan.setRouteCarriedTraffic(routeId, h_d * f_p);
				}
			}
		}
	}
}
